@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -21,8 +21,8 @@
 
 #include <drivers/atmodem/atutil.h>
 
-extern void calypso_voicecall_init();
-extern void calypso_voicecall_exit();
+extern void calypso_voicecall_init(void);
+extern void calypso_voicecall_exit(void);
 
-extern void calypso_stk_init();
-extern void calypso_stk_exit();
+extern void calypso_stk_init(void);
+extern void calypso_stk_exit(void);
