@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -490,7 +489,7 @@ error:
 	g_free(cbd);
 }
 
-static struct ofono_gprs_driver driver = {
+static const struct ofono_gprs_driver driver = {
 	.name			= "isimodem",
 	.probe			= isi_gprs_probe,
 	.remove			= isi_gprs_remove,
