@@ -305,8 +305,9 @@ static void at_command_destroy(struct at_command *cmd)
 	g_free(cmd);
 }
 
-static void free_terminator(struct terminator_info *info)
+static void free_terminator(gpointer pointer)
 {
+	struct terminator_info *info = pointer;
 	g_free(info->terminator);
 	info->terminator = NULL;
 	g_free(info);
@@ -325,8 +326,7 @@ static void chat_cleanup(struct at_chat *chat)
 	chat->command_queue = NULL;
 
 	/* Cleanup any response lines we have pending */
-	g_slist_foreach(chat->response_lines, (GFunc)g_free, NULL);
-	g_slist_free(chat->response_lines);
+	g_slist_free_full(chat->response_lines, g_free);
 	chat->response_lines = NULL;
 
 	/* Cleanup registered notifications */
@@ -357,9 +357,7 @@ static void chat_cleanup(struct at_chat *chat)
 	chat->syntax = NULL;
 
 	if (chat->terminator_list) {
-		g_slist_foreach(chat->terminator_list,
-					(GFunc)free_terminator, NULL);
-		g_slist_free(chat->terminator_list);
+		g_slist_free_full(chat->terminator_list, free_terminator);
 		chat->terminator_list = NULL;
 	}
 }
@@ -461,8 +459,7 @@ static void at_chat_finish_command(struct at_chat *p, gboolean ok, char *final)
 		cmd->callback(ok, &result, cmd->user_data);
 	}
 
-	g_slist_foreach(response_lines, (GFunc)g_free, NULL);
-	g_slist_free(response_lines);
+	g_slist_free_full(response_lines, g_free);
 
 	g_free(final);
 	at_command_destroy(cmd);
@@ -583,7 +580,7 @@ static void have_line(struct at_chat *p, char *str)
 		return;
 
 	/* Check for echo, this should not happen, but lets be paranoid */
-	if (!strncmp(str, "AT", 2) == TRUE)
+	if (!strncmp(str, "AT", 2))
 		goto done;
 
 	cmd = g_queue_peek_head(p->command_queue);
@@ -1135,6 +1132,29 @@ static gboolean at_chat_cancel_group(struct at_chat *chat, guint group)
 	return TRUE;
 }
 
+static gpointer at_chat_get_userdata(struct at_chat *chat,
+						guint group, guint id)
+{
+	GList *l;
+	struct at_command *c;
+
+	if (chat->command_queue == NULL)
+		return NULL;
+
+	l = g_queue_find_custom(chat->command_queue, GUINT_TO_POINTER(id),
+				at_command_compare_by_id);
+
+	if (l == NULL)
+		return NULL;
+
+	c = l->data;
+
+	if (c->gid != group)
+		return NULL;
+
+	return c->user_data;
+}
+
 static guint at_chat_register(struct at_chat *chat, guint group,
 				const char *prefix, GAtNotifyFunc func,
 				gboolean expect_pdu, gpointer user_data,
@@ -1540,6 +1560,14 @@ gboolean g_at_chat_cancel_all(GAtChat *chat)
 	return at_chat_cancel_group(chat->parent, chat->group);
 }
 
+gpointer g_at_chat_get_userdata(GAtChat *chat, guint id)
+{
+	if (chat == NULL)
+		return NULL;
+
+	return at_chat_get_userdata(chat->parent, chat->group, id);
+}
+
 guint g_at_chat_register(GAtChat *chat, const char *prefix,
 				GAtNotifyFunc func, gboolean expect_pdu,
 				gpointer user_data,
