@@ -47,6 +47,7 @@
 #include <ofono/radio-settings.h>
 #include <ofono/log.h>
 #include <ofono/location-reporting.h>
+#include <ofono/sim-auth.h>
 
 #include <drivers/atmodem/atutil.h>
 #include <drivers/atmodem/vendor.h>
@@ -137,7 +138,9 @@ static void sim_state_cb(gboolean present, gpointer user_data)
 	at_util_sim_state_query_free(data->sim_state_query);
 	data->sim_state_query = NULL;
 
-	data->flags |= MBM_FLAG_HAVE_SIM;
+	if (present)
+		data->flags |= MBM_FLAG_HAVE_SIM;
+
 	ofono_modem_set_powered(modem, TRUE);
 }
 
@@ -205,6 +208,14 @@ static void cfun_query(gboolean ok, GAtResult *result, gpointer user_data)
 
 	g_at_result_iter_next_number(&iter, &status);
 
+	g_at_chat_send(data->modem_port, "AT&F E0 V1 X4 &C0 +CMEE=1", NULL,
+					NULL, NULL, NULL);
+	g_at_chat_send(data->data_port, "AT&F E0 V1 X4 &C0 +CMEE=1", NULL,
+					NULL, NULL, NULL);
+
+	g_at_chat_send(data->modem_port, "AT*E2CFUN=1", none_prefix,
+					NULL, NULL, NULL);
+
 	if (status != 4) {
 		g_at_chat_send(data->modem_port, "AT+CFUN=4", none_prefix,
 				cfun_enable, modem, NULL);
@@ -214,6 +225,15 @@ static void cfun_query(gboolean ok, GAtResult *result, gpointer user_data)
 	cfun_enable(TRUE, NULL, modem);
 }
 
+static void epev_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct ofono_sim *sim = ofono_modem_get_sim(modem);
+
+	if (sim)
+		ofono_sim_initialized_notify(sim);
+}
+
 static void emrdy_notifier(GAtResult *result, gpointer user_data)
 {
 	struct ofono_modem *modem = user_data;
@@ -252,6 +272,10 @@ static void emrdy_query(gboolean ok, GAtResult *result, gpointer user_data)
 	if (ok)
 		return;
 
+	/* Sometimes we query EMRDY just as the EMRDY notifier is fired */
+	if (data->flags & MBM_FLAG_SAW_EMRDY)
+		return;
+
 	/* On some MBM hardware the EMRDY cannot be queried, so if this fails
 	 * we try to run CFUN? to check the state.  CFUN? will fail unless
 	 * EMRDY: 1 has been sent, in which case the emrdy_notifier should be
@@ -329,16 +353,13 @@ static int mbm_enable(struct ofono_modem *modem)
 	g_at_chat_register(data->modem_port, "*EMRDY:", emrdy_notifier,
 					FALSE, modem, NULL);
 
-	g_at_chat_send(data->modem_port, "AT&F E0 V1 X4 &C0 +CMEE=1", NULL,
-					NULL, NULL, NULL);
-	g_at_chat_send(data->data_port, "AT&F E0 V1 X4 &C0 +CMEE=1", NULL,
-					NULL, NULL, NULL);
-
-	g_at_chat_send(data->modem_port, "AT*E2CFUN=1", none_prefix,
-					NULL, NULL, NULL);
 	g_at_chat_send(data->modem_port, "AT*EMRDY?", none_prefix,
 				emrdy_query, modem, NULL);
 
+	g_at_chat_send(data->modem_port, "AT*EPEE=1", NULL, NULL, NULL, NULL);
+	g_at_chat_register(data->modem_port, "*EPEV", epev_notify,
+					FALSE, modem, NULL);
+
 	return -EINPROGRESS;
 }
 
@@ -431,6 +452,7 @@ static void mbm_post_sim(struct ofono_modem *modem)
 	ofono_radio_settings_create(modem, 0, "stemodem", data->modem_port);
 
 	ofono_sms_create(modem, 0, "atmodem", data->modem_port);
+	ofono_sim_auth_create(modem);
 }
 
 static void mbm_post_online(struct ofono_modem *modem)
