@@ -76,8 +76,6 @@ DBusMessage *__ofono_error_from_error(const struct ofono_error *error,
 
 void __ofono_dbus_pending_reply(DBusMessage **msg, DBusMessage *reply);
 
-gboolean __ofono_dbus_valid_object_path(const char *path);
-
 struct ofono_watchlist_item {
 	unsigned int id;
 	void *notify;
@@ -153,6 +151,8 @@ enum ofono_atom_type {
 	OFONO_ATOM_TYPE_HANDSFREE,
 	OFONO_ATOM_TYPE_SIRI,
 	OFONO_ATOM_TYPE_NETMON,
+	OFONO_ATOM_TYPE_LTE,
+	OFONO_ATOM_TYPE_IMS,
 };
 
 enum ofono_atom_watch_condition {
@@ -368,6 +368,14 @@ unsigned short __ofono_sms_get_next_ref(struct ofono_sms *sms);
 
 #include <ofono/sim.h>
 
+struct ofono_sim_aid_session;
+enum sim_app_type;
+
+typedef void (*ofono_sim_session_event_cb_t)(ofono_bool_t active,
+		int session_id, void *data);
+
+ofono_bool_t __ofono_sim_ust_service_available(struct ofono_sim *sim,
+						int ust_service);
 ofono_bool_t __ofono_sim_service_available(struct ofono_sim *sim,
 						int ust_service,
 						int sst_service);
@@ -385,6 +393,32 @@ void __ofono_sim_refresh(struct ofono_sim *sim, GSList *file_list,
 
 void __ofono_sim_recheck_pin(struct ofono_sim *sim);
 
+GSList *__ofono_sim_get_aid_list(struct ofono_sim *sim);
+
+unsigned int __ofono_sim_add_session_watch(
+		struct ofono_sim_aid_session *session,
+		ofono_sim_session_event_cb_t notify, void *data,
+		ofono_destroy_func destroy);
+
+void __ofono_sim_remove_session_watch(struct ofono_sim_aid_session *session,
+		unsigned int id);
+
+struct ofono_sim_aid_session *__ofono_sim_get_session_by_aid(
+		struct ofono_sim *sim, unsigned char *aid);
+
+struct ofono_sim_aid_session *__ofono_sim_get_session_by_type(
+		struct ofono_sim *sim, enum sim_app_type type);
+
+int __ofono_sim_session_get_id(struct ofono_sim_aid_session *session);
+
+enum sim_app_type __ofono_sim_session_get_type(
+		struct ofono_sim_aid_session *session);
+
+unsigned char *__ofono_sim_session_get_aid(
+		struct ofono_sim_aid_session *session);
+
+const char *__ofono_sim_get_impi(struct ofono_sim *sim);
+
 #include <ofono/stk.h>
 
 typedef void (*__ofono_sms_sim_download_cb_t)(ofono_bool_t ok,
@@ -532,3 +566,5 @@ ofono_bool_t __ofono_private_network_request(ofono_private_network_cb_t cb,
 						int *id, void *data);
 
 #include <ofono/netmon.h>
+#include <ofono/lte.h>
+#include <ofono/ims.h>
